%**************************************************************************
%**************************************************************************
%This M code is designed to obtain daily NDVI time series with high quality
%by combining the daily and composite products of MODIS.
%In this code, the daily product is MOD09GQ product, the composite product
%is MOD09Q1 (8-day composition) or MOD13Q1 (16-day composite) product.
%This code is designed by:
%------------Shun Hu, hushun@cug.edu.cn, CUG, Wuhan, China-----------------
%The related reference is:Linglin Zeng, Brain Wardlow, Shun Hu*, et al., 2021. %A novel strategy to reconstruct NDVI time-series with high temporal 
%resolution from MODIS multi-temporal composite products.
%**************************************************************************
%**************************************************************************

%%
%Clear the old workspace.
clear
clc

%%
%The time length of daily observations
tinit = 1;
tend = 4383;

%%
%The time interval of composite product (8 or 16 in here).
D = 8;

%%
% Parameter values' setting
CMFR = 0.05;
CRu = 0.05;
CRd = 0.05;
CRatu = 20;
CRatd = 20;

GQMFR = 0.05;
GQRu = 0.05;
GQRd = 0.025;
GQRatu = 10;
GQRatd = 5;

FSC = 3; %3 or 5 for 8-day; 2 or 3 for 16-day 
FSE = 5;%5 or 10

%%
% Import original data
ONDVIC = dlmread('OriginalCompositeData.txt');
ONDVIGQ = dlmread('OringinalDailyData.txt');
Judge = find(isnan(ONDVIC(:,2)));
ONDVIC(Judge,:) = [];
Judge = [];

%%
%Step 1: Computing the actual qcquisition date of composite data
for m = 1:length(ONDVIC(:,1))
    num = 0;
    for n = 1:length(ONDVIGQ(:,1))
        if(ONDVIGQ(n,1)>=ONDVIC(m,1)&...
                ONDVIGQ(n,1)<ONDVIC(m,1)+D)
            num = num+1;
            tempdat(num,1:2) = ONDVIGQ(n,1:2);
            tempdat(num,3) = abs(ONDVIGQ(n,2)-ONDVIC(m,2));
        end
    end
    [tempmin,id] = min(tempdat(:,3));
    ANDVIC(m,1:2) = tempdat(id,1:2);
    num = [];
    tempdat = [];
    tempmin = [];
    id = [];
end
judge = find(ONDVIGQ(:,2)<0|ONDVIGQ(:,2)>1);
ONDVIGQ(judge,:) = [];
judge = [];
judge = find(isnan(ONDVIGQ(:,2)));
ONDVIGQ(judge,:) = [];
judge = [];
AllONDVIGQ(tinit:tend,1:2) = -99.99;
AllONDVIGQ(tinit:tend,1) = [tinit:1:tend]';
AllONDVIGQ(ONDVIGQ(:,1),2) = ONDVIGQ(:,2);
Judge = find(isnan(ANDVIC(:,2)));
if(~isempty(Judge))
    ANDVIC(Judge,:) = [];
    Judge = [];
end
Judge = find(ANDVIC(:,2)<0|ANDVIC(:,2)>1);
if(~isempty(Judge))
    ANDVIC(Judge,:) = [];
    Judge = [];
end
num = 0;
for m = 1:length(ANDVIC(:,1))-1
    if(ANDVIC(m+1,1)==ANDVIC(m,1))
        num = num+1;
        Judge(num,1) = m;
    end
end
if(num>0)
    ANDVIC(Judge,:) = []; 
    Judge = [];
    num = [];
end

%%
% Step 2-4: Finding out the reasonable NDVIAD
tmpANDVIC = ANDVIC;
for i = 2:length(ANDVIC(:,1))-1
    if(ANDVIC(i,2)<ANDVIC(i-1,2)&ANDVIC(i,2)<ANDVIC(i+1,2))
        tmpANDVIC(i,2) = NaN;
    end
end
Judge = find(isnan(tmpANDVIC(:,2)));
tmpANDVIC(Judge,:) = [];
Judge = [];
figure(2)
subplot(2,1,1)
plot(ANDVIC(:,1),ANDVIC(:,2),'ko');
hold on
POC = 1; 
testF(:,1) = tmpANDVIC(:,1);
testF(:,2) = smooth(tmpANDVIC(:,1),tmpANDVIC(:,2),FSC,'sgolay',POC);
NDVIP(:,1) = [tinit:1:tend]';
NDVIP(:,2) = interp1(testF(:,1),testF(:,2),NDVIP(:,1),'phcip');
plot(NDVIP(:,1),NDVIP(:,2),'k-','LineWidth',1.5);
hold on
for jiter = 1:100
    EANDVIC = [];
    testF2 = [];
    DNDVIP = NDVIP;
    for i = 2:length(NDVIP(:,1))-1
        DNDVIP(i,2) = abs(NDVIP(i+1,2)-NDVIP(i-1,2))/2;
    end
    DNDVIP(1,2) = DNDVIP(2,2);
    DNDVIP(end,2) = DNDVIP(end-1,2);
    num = 0;
    meanNDVIP = mean(NDVIP(:,2));
    stdNDVIP = std(NDVIP(:,2));

    maxp = max(NDVIP(:,2));
    minp = min(NDVIP(:,2));

    for m = 1:length(ANDVIC(:,1))
        for n = 1:length(NDVIP(:,1))
            if(ANDVIC(m,1)==NDVIP(n,1))
                num = num+1;
                EANDVIC(num,1:2) = ANDVIC(m,1:2);

                if(stdNDVIP<CRu)
                    uplimit = CRu+CRatu*DNDVIP(n,2);
                    bottomlimit = -CRd-CRatd*DNDVIP(n,2);
                else
                    uplimit = CRu+CMFR*((NDVIP(n,2)-minp)/(maxp-minp))...
                        +CRatu*DNDVIP(n,2);
                    bottomlimit = -CRd-CRatd*DNDVIP(n,2);
                end

                if(EANDVIC(num,2)-NDVIP(n,2)>uplimit|...
                        EANDVIC(num,2)-NDVIP(n,2)<bottomlimit)
                    EANDVIC(num,2) = -100;
                end
            end
        end
    end
    Judge = find(EANDVIC(:,2)<0|EANDVIC(:,2)>1);
    if(~isempty(Judge))
        EANDVIC(Judge,:) = [];
        Judge = [];
    end
    if(jiter>=2)
        if(length(tmpEANDVIC(:,1))~=length(EANDVIC(:,1)))
            tmpEANDVIC = [];
        else
            JudgeEA = sum(abs(tmpEANDVIC(:,2)-EANDVIC(:,2)));
            if(JudgeEA==0.0)
                break;
            end
        end
    end
    POC = 1;
    testF2(:,1) = EANDVIC(:,1);
    testF2(:,2) = smooth(EANDVIC(:,1),EANDVIC(:,2),FSC,'sgolay',POC);
    NDVIP(:,1) = [tinit:1:tend]';
    NDVIP(:,2) = interp1(testF2(:,1),testF2(:,2),NDVIP(:,1),'phcip');
    tmpEANDVIC = EANDVIC;
end
plot(EANDVIC(:,1),EANDVIC(:,2),'go');
hold on
plot(NDVIP(:,1),NDVIP(:,2),'g-','LineWidth',1.5);
hold off
set(gca,'XLim',[tinit tend]);
set(gca,'YLim',[0,1]);
legend('Original NDVIC','Initial NDVIP','Reasonable NDVIC','FindNDVIP');

%%
% Finding out the effective ENDVIGQ
DNDVIP = NDVIP;
for i = 2:length(NDVIP(:,1))-1
    DNDVIP(i,2) = abs(NDVIP(i+1,2)-NDVIP(i-1,2))/2;
end
DNDVIP(1,2) = DNDVIP(2,2);
DNDVIP(end,2) = DNDVIP(end-1,2);

num = 0;
meanNDVIP = mean(NDVIP(:,2));
stdNDVIP = std(NDVIP(:,2));

maxp = max(NDVIP(:,2));
minp = min(NDVIP(:,2));

for m = 1:length(ONDVIGQ(:,1))
    for n = 1:length(NDVIP(:,1))
        if(ONDVIGQ(m,1)==NDVIP(n,1))
            num = num+1;
            ENDVIGQ(num,1:2) = ONDVIGQ(m,1:2);
            
            if(stdNDVIP<GQRu) %The composite has small vairation
                uplimit = GQRu+GQRatu*DNDVIP(n,2);
                bottomlimit = -GQRd-GQRatd*DNDVIP(n,2);
            else
                uplimit = GQRu+GQMFR*((NDVIP(n,2)-minp)/(maxp-minp))...
                    +GQRatu*DNDVIP(n,2);
                bottomlimit = -GQRd-GQRatd*DNDVIP(n,2);
            end

            if(ENDVIGQ(num,2)-NDVIP(n,2)>uplimit|...
                    ENDVIGQ(num,2)-NDVIP(n,2)<bottomlimit)
                ENDVIGQ(num,2) = -100;
            end
        end
    end
end
Judge = find(ENDVIGQ(:,2)<0|ENDVIGQ(:,2)>1);
if(~isempty(Judge))
    ENDVIGQ(Judge,:) = [];
    Judge = [];
end
for i = 1:length(EANDVIC(:,1))
    for j = 1:length(ENDVIGQ(:,1))
        if(EANDVIC(i,1)==ENDVIGQ(j,1))
            ENDVIGQ(j,2) = EANDVIC(i,2);
        end
    end
end
ENDVIGQ2(:,1) = [ENDVIGQ(:,1)];
ENDVIGQ2(:,2) = [ENDVIGQ(:,2)];
ENDVIGQ2 = sortrows(ENDVIGQ2,1);
for i = 1:length(ENDVIGQ2(:,1))-1
    if(ENDVIGQ2(i,1)==ENDVIGQ2(i+1,1))
        ENDVIGQ2(i+1,2) = NaN;
    end
end
Judge = find(isnan(ENDVIGQ2(:,2)));
ENDVIGQ2(Judge,:) = [];
Judge = [];

%%
% Reconstruction the daily NDVI time series
POE = 1; 

subplot(2,1,2)
plot(ONDVIGQ(:,1),ONDVIGQ(:,2),'ko')
hold on
plot(ENDVIGQ2(:,1),ENDVIGQ2(:,2),'ro')
hold on

FSE = 5;
NDVIFsg3(:,1) = ENDVIGQ2(:,1);
NDVIFsg3(:,2) = smooth(ENDVIGQ2(:,1),ENDVIGQ2(:,2),FSE,'sgolay',1);
NDVIPsg3(:,1) = [tinit:1:tend]';
NDVIPsg3(:,2) = interp1(NDVIFsg3(:,1),NDVIFsg3(:,2),NDVIPsg3(:,1),'phcip');
DMFSE5 = NDVIPsg3;
plot(DMFSE5(:,1),DMFSE5(:,2),'b-','linewidth',1.5);
hold on
FSE = 10;
NDVIFsg3(:,1) = ENDVIGQ2(:,1);
NDVIFsg3(:,2) = smooth(ENDVIGQ2(:,1),ENDVIGQ2(:,2),FSE,'sgolay',1);
NDVIPsg3(:,1) = [tinit:1:tend]';
NDVIPsg3(:,2) = interp1(NDVIFsg3(:,1),NDVIFsg3(:,2),NDVIPsg3(:,1),'phcip');
DMFSE10 = NDVIPsg3;
plot(DMFSE10(:,1),DMFSE10(:,2),'g-','linewidth',1.5);
hold off
set(gca,'XLim',[tinit tend]);
set(gca,'YLim',[0,1]);
legend('Original NDVIGQ','Effective NDVIGQ','DMFSC3FSE5','DMFSC3FSE10');
set(gcf,'position',[40,100,2000,800])